/*
 * linux/include/asm-arm/arch-ep93xx/fb.h
 *
 * Copyright 2009 Atmark Techno, Inc. All Rights Reserved.
 *
 * Based on the Cirrus Logic ep93xxfb driver.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */

#ifndef __ASM_ARCH_FB_H__
#define __ASM_ARCH_FB_H__

#include <linux/fb.h>

#ifdef __KERNEL__

#define POL_HIGH		1
#define POL_LOW        		0
#define EDGE_RISING     	1
#define EDGE_FALLING    	0
#define CLK_INTERNAL    	1
#define CLK_EXTERNAL    	0

#define CRT_OUT         	0
#define LCD_OUT         	1
#define NTSC_OUT          	2
#define PAL_OUT          	3

#define MAX_XRES                1280
#define MAX_YRES                1024
#define MAX_BPP                 32
#define MAX_FBMEM_SIZE          3686400/*1920000*/

#define MAX_XRES_CRT            MAX_XRES
#define MAX_YRES_CRT            MAX_YRES
#define MAX_XRES_SVIDEO         1024
#define MAX_YRES_SVIDEO         768

#define PIXEL_FORMAT_SHIFT      17
#define PIXEL_FORMAT_4          (1 << PIXEL_FORMAT_SHIFT)
#define PIXEL_FORMAT_8          (2 << PIXEL_FORMAT_SHIFT)
#define PIXEL_FORMAT_16         (4 << PIXEL_FORMAT_SHIFT)
#define PIXEL_FORMAT_24         (6 << PIXEL_FORMAT_SHIFT)
#define PIXEL_FORMAT_32         (7 << PIXEL_FORMAT_SHIFT)


struct ep93xxfb_videomodes
{
	const char *name[8];

	unsigned long hres; 	// Horizontal Valid
	unsigned long hfp;	// Horizontal Front Porch
	unsigned long hsync;	// Horizontal Sync Width
	unsigned long hbp;	// Horizontal Back Porch

	unsigned long vres;	// Vertical Valid
	unsigned long vfp;	// Vertical Front Porch
	unsigned long vsync;	// Vertical Sync Width
	unsigned long vbp;	// Vertical Back Porch

	unsigned long refresh;	// Vertical Sync Frequency

	unsigned long clk_src:1;
	unsigned long clk_edge:1;
	unsigned long pol_blank:1;
	unsigned long pol_hsync:1;
	unsigned long pol_vsync:1;
};

struct ep93xxfb_info
{
	dma_addr_t	fb_phys;
	void		*fb_log;
	unsigned long	fb_size;
	unsigned long	fb_actsize;

	unsigned long	xtotal;
	unsigned long	ytotal;

	unsigned int	xres;
	unsigned int	xfp;
	unsigned int	xsync;
	unsigned int	xbp;

	unsigned int	yres;
	unsigned int	yfp;
	unsigned int	ysync;
	unsigned int	ybp;
	unsigned int	bpp;
    
	unsigned long	refresh;
	unsigned long	pixclock;
	unsigned long	pixformat;

	unsigned int	clk_src;
	unsigned int	clk_edge;
	unsigned int	pol_blank;
	unsigned int	pol_xsync;
	unsigned int	pol_ysync;

	unsigned char	automods;

	void (*configure)(unsigned char value);
	void (*on)(unsigned char value);
	void (*off)(unsigned char value);
};

#endif /* __KERNEL__ */

/* static int ep93xxfb_setclk(void); */
/* static int ep93xx_get_max_video_clk(void); */
/* static void ep93xxfb_set_pixelmode(int bpp); */
/* static void ep93xxfb_timing_signal_generation(void); */
/* static int  ep93xxfb_blank(int blank_mode,struct fb_info *info); */

#define EE_DELAY_USEC                   2
#define EE_READ_TIMEOUT                 100
#define CX25871_DEV_ADDRESS             0x88
#define GPIOG_EEDAT   			2
#define GPIOG_EECLK     		1
#define CXMODES_COUNT   		24

struct cx25871_vmodes
{
	const char	*name;
	unsigned char	automode;
	unsigned int	hres;
	unsigned int	vres;
	unsigned int	hclktotal;
	unsigned int	vclktotal;
	unsigned int	hblank;
	unsigned int	vblank;
	unsigned long	clkfrequency;
};

/* static int write_reg(unsigned char ucRegAddr, unsigned char ucRegValue); */
/* static void cx25871_on(unsigned char value); */
/* static void cx25871_off(unsigned char value); */
/* static void cx25871_config(unsigned char value); */

/* static void philips_lb064v02_on(unsigned char value); */
/* static void philips_lb064v02_off(unsigned char value); */


/*
 * ioctl(fd, FBIO_EP93XX_GET_CAPS, unsigned long *)
 *
 * The unsigned long is filled in with these capabilities flags to indicate
 * what the frame buffer driver is capable of performing.
 */

#define EP93XX_CAP_CURSOR	(1<<0)
#define EP93XX_CAP_LINE		(1<<1)
#define EP93XX_CAP_FILL		(1<<2)
#define EP93XX_CAP_BLIT		(1<<3)
#define EP93XX_CAP_COPY		(1<<4)

/*
 * ioctl(fd, FBIO_EP93XX_CURSOR, ep93xx_cursor *)
 *
 * "data" points to an array of pixels that define the cursor; each row should
 * be a multiple of 32-bit values (i.e. 16 pixels).  Each pixel is two bits,
 * where the values are:
 *
 *     00 => transparent    01 => invert    10 => color1    11 => color2
 *
 * The data is arranged as follows (per word):
 *
 *    bits: |31-30|29-28|27-26|25-24|23-22|21-20|19-18|17-16|
 *   pixel: | 12  | 13  | 14  | 15  |  8  |  9  | 10  | 11  |
 *    bits: |15-14|13-12|11-10| 9-8 | 7-6 | 5-4 | 3-2 | 1-0 |
 *   pixel: |  4  |  5  |  6  |  7  |  0  |  1  |  2  |  3  |
 *
 * Regardless of the frame buffer color depth, "color1", "color2",
 * "blinkcolor1", and "blinkcolor2" are 24-bit colors since the cursor is
 * injected into the data stream right before the video DAC.
 *
 * When "blinkrate" is not zero, pixel value 10 will alternate between "color1"
 * and "blinkcolor1" (similar for pixel value 11 and "color2"/"blinkcolor2").
 *
 * "blinkrate" ranges between 0 and 255.  When 0, blinking is disabled.  255 is
 * the fastest blink rate and 1 is the slowest.
 *
 * Both "width" and "height" must be between 1 and 64; it is preferable to have
 * "width" a multiple of 16.
 */

#define CURSOR_BLINK            (1<<0)
#define CURSOR_MOVE             (1<<1)
#define CURSOR_SETSHAPE         (1<<2)
#define CURSOR_SETCOLOR         (1<<3)
#define CURSOR_ON               (1<<4)
#define CURSOR_OFF              (1<<5)

struct ep93xx_cursor {
	unsigned long flags;
	unsigned long dx;           // Only used if CURSOR_MOVE is set
	unsigned long dy;           // Only used if CURSOR_MOVE is set
	unsigned long width;        // Only used if CURSOR_SETSHAPE is set
	unsigned long height;       // Only used if CURSOR_SETSHAPE is set
	const char    *data;	    // Only used if CURSOR_SETSHAPE is set
	unsigned long blinkrate;    // Only used if CURSOR_BLINK is set
	unsigned long color1;       // Only used if CURSOR_SETCOLOR is set
	unsigned long color2;       // Only used if CURSOR_SETCOLOR is set
	unsigned long blinkcolor1;  // Only used if CURSOR_SETCOLOR is set
	unsigned long blinkcolor2;  // Only used if CURSOR_SETCOLOR is set
};


/*
 * ioctl(fd, FBIO_EP93XX_FILL, ep93xx_fill *)
 *
 * Fills from dx to (dx + width - 1), and from dy to (dy + height - 1).
 */
struct ep93xx_fill {
    __u32 dx;
    __u32 dy;
    __u32 width;
    __u32 height;
    __u32 color;
};

/*
 * The bits in the flags field of ep93xx_line.
 *
 * ioctl(fd, FBIO_EP93XX_LINE, ep93xx_line *)
 *
 * The line starts at ("x1","y1") and ends at ("x2","y2").  This means that
 * when using a pattern, the two coordinates are not transitive (i.e. swapping
 * ("x1","y1") with ("x2","y2") will not necessarily draw the exact same line,
 * pattern-wise).
 *
 * "pattern" is a 2 to 16 bit pattern (since a 1 bit pattern isn't much of a
 * pattern).  The lower 16 bits define the pattern (1 being foreground, 0 being
 * background or transparent), and bits 19-16 define the length of the pattern
 * (as pattern length - 1).  So, for example, "0xf00ff" defines a 16 bit
 * with the first 8 pixels in the foreground color and the next 8 pixels in the
 * background color or transparent.
 *
 * LINE_PRECISE is used to apply angularly corrected patterns to line.  It
 * should only be used when LINE_PATTERN is also set.  The pattern will be
 * applied along the length of the line, instead of along the length of the
 * major axis.  This may result in the loss of fine details in the pattern, and
 * will take more time to draw the line in most cases.
 */

#define LINE_PATTERN	(1<<0)
#define LINE_PRECISE	(1<<1)
#define LINE_BACKGROUND	(1<<2)

struct ep93xx_line {
	unsigned long flags;
	unsigned long x1;
	unsigned long y1;
	unsigned long x2;
	unsigned long y2;
	unsigned long fgcolor;
	unsigned long bgcolor;      // Only used if LINE_BACKGROUND is set
	unsigned long pattern;      // Only used if LINE_PATTERN is set
};

/*
 * ioctl(fd, FBIO_EP93XX_BLIT, ep93xx_blit *)
 *
 * When BLIT_SOURCE_MEMORY is set, "data" points to an array of pixels that are
 * to be blitted to the screen; each row should be a multiple of 32-bit values.
 * When BLIT_1BPP_SOURCE is set, each pixel is one bit and is arranged as
 * follows (per word):
 *
 *     bit: |31|30|29|28|27|26|25|24|23|22|21|20|19|18|17|16|
 *   pixel: |24|25|26|27|28|29|30|31|16|17|18|19|20|21|22|23|
 *     bit: |15|14|13|12|11|10| 9| 8| 7| 6| 5| 4| 3| 2| 1| 0|
 *   pixel: | 8| 9|10|11|12|13|14|15| 0| 1| 2| 3| 4| 5| 6| 7|
 *
 * When the frame buffer is in 8bpp mode, each pixel is one byte and is
 * arranged as follows (per word):
 *
 *    bits: |31-24|23-16|15-8 | 7-0 |
 *   pixel: |  3  |  2  |  1  |  0  |
 *
 * When the frame buffer is in 16bpp mode, each pixel is two bytes and is
 * arranged as follows (per word):
 *
 *    bits: |31-16|15-0 |
 *   pixel: |  1  |  0  |
 *
 * When BLIT_SOURCE_SCREEN is set, BLIT_1BPP_SOURCE is not valid (not for any
 * hardware imposed reason, but because it simply doesn't make any sense).
 * When BLIT_SOURCE_MEMORY and BLIT_1BPP_SOURCE are set, BLIT_MASK_DISABLE must
 * be set.
 *
 * BLIT_MASK_MASK applies a masking operation on the source data as it is read.
 * When not disabled, each pixel is modified by the given operation with the
 * "fgcolor" value.
 *
 * BLIT_DEST_MASK determines how the data is written to the destination
 * location.  When not disabled, each destination pixel is modified by the
 * given operation with the current value.  This operation takes place after
 * the BLIT_MASK_MASK operation.
 *
 * BLIT_TRANSPARENT allows a transparency color to be defined for the blit.  If
 * set, after BLIT_DEST_MASK has been applied, if the resulting pixel matches
 * "transcolor", the destination pixel is not modified.
 */

#define BLIT_SOURCE_MASK	0x00000001
#define BLIT_SOURCE_MEMORY	0x00000000
#define BLIT_SOURCE_SCREEN	0x00000001
#define BLIT_TRANSPARENT	0x00000004
#define BLIT_MASK_MASK		0x00000600
#define BLIT_MASK_DISABLE	0x00000000
#define BLIT_MASK_AND		0x00000200
#define BLIT_MASK_OR		0x00000400
#define BLIT_MASK_XOR		0x00000600
#define BLIT_DEST_MASK		0x00001800
#define BLIT_DEST_DISABLE	0x00000000
#define BLIT_DEST_AND		0x00000800
#define BLIT_DEST_OR		0x00001000
#define BLIT_DEST_XOR		0x00001800
#define BLIT_1BPP_SOURCE	0x00006000

struct ep93xx_blit {
    __u32 flags;
    __u32 dx;
    __u32 dy;
    __u32 width;
    __u32 height;
    const char *data;	// Only used if BLIT_SOURCE_MEMORY is set
    __u32 swidth;	// Only used if BLIT_SOURCE_MEMORY is set
    __u32 sx;		// Only used if BLIT_SOURCE_SCREEN is set
    __u32 sy;		// Only used if BLIT_SOURCE_SCREEN is set
    __u32 fgcolor;	// Only used if BLIT_1BPP_SOURCE or BLIT_MASK_?? is set
    __u32 bgcolor;	// Only used if BLIT_1BPP_SOURCE is set
    __u32 transcolor;	// Only used if BLIT_TRANSPARENT is set
};

/* ioctl command */
#define FBIO_EP93XX_IOCTL_BASE	'e'
#define FBIO_EP93XX_GET_CAPS	_IOR(FBIO_EP93XX_IOCTL_BASE, 0x01, unsigned long *)
#define FBIO_EP93XX_CURSOR	_IOW(FBIO_EP93XX_IOCTL_BASE, 0x02, struct ep93xx_cursor *)
#define FBIO_EP93XX_LINE	_IOW(FBIO_EP93XX_IOCTL_BASE, 0x03, struct ep93xx_line *)
#define FBIO_EP93XX_FILL	_IOW(FBIO_EP93XX_IOCTL_BASE, 0x04, struct ep93xx_fill *)
#define FBIO_EP93XX_BLIT	_IOW(FBIO_EP93XX_IOCTL_BASE, 0x05, struct ep93xx_blit *)
#define FBIO_EP93XX_COPY	_IOW(FBIO_EP93XX_IOCTL_BASE, 0x06, struct fb_copyarea *)

#ifdef __KERNEL__

#ifdef CONFIG_EP93XX_GRAPHICS
extern int ep93xxfb_request_irq(unsigned int irq);
extern void ep93xxfb_free_irq(unsigned int irq);
extern int ep93xxfb_line(struct fb_info *info, struct ep93xx_line *line);
extern int ep93xxfb_fillrect(struct fb_info *info, struct ep93xx_fill *fill);
extern int ep93xxfb_imageblit(struct fb_info *info, struct ep93xx_blit *blit);
extern void ep93xxfb_copyarea(struct fb_info *p, const struct fb_copyarea *area);
#else
static inline int ep93xxfb_request_irq(void) { return 0; }
static inline void ep93xxfb_free_irq(unsigned int) {}
static inline int ep93xxfb_line(struct fb_info *info, struct ep93xx_line *line) { return 0; }
static inline int ep93xxfb_fillrect(struct fb_info *info, struct ep93xx_fill *fill) { return 0; }
static inline int ep93xxfb_imageblit(struct fb_info *info, struct ep93xx_blit *blit) { return 0; }
static inline void ep93xxfb_copyarea(struct fb_info *p, const struct fb_copyarea *area) {}
#endif /* CONFIG_EP93XX_GRAPHICS */

static inline int ep93xx_have_graphics_accelerator(void)
{
#ifdef CONFIG_EP93XX_GRAPHICS
	return 1;
#else
	return 0;
#endif /* CONFIG_EP93XX_GRAPHICS */
}

extern unsigned int pseudo_palette[256];
extern unsigned long *cursor_data;
extern struct ep93xxfb_info epinfo;

extern int vout;
extern char *vmode;
extern int vmodeidx;
extern int depth;
extern int pixel_mask;
extern int pixel_shift;
extern int pixel_format;

extern struct cx25871_vmodes cx_ntscmodes[];
extern int cx_ntscmodes_sz;
extern struct cx25871_vmodes cx_palmodes[];
extern int cx_palmodes_sz;
extern struct ep93xxfb_videomodes ep93xxfb_crtmodes[];
extern int ep93xxfb_crtmodes_sz;
extern struct ep93xxfb_videomodes ep93xxfb_lcdmodes[];
extern int ep93xxfb_lcdmodes_sz;

#endif /* __KERNEL__ */

#endif /* __ASM_ARCH_FB_H__ */

